#!/bin/python

import re, whrandom, copy
from threading import *

def cifind(a, b): return a.lower().find(b.lower()) != -1

class Song:
    def __init__(self, id, server, filename, title, artist):
        self.set_played(0)
        self.set_id(id)
        self.server = server
        self.filename = filename
        self.title = title
        self.artist = artist

    def was_played(self): return self.played
    def set_played(self, value): self.played = value
    def get_server(self): return self.server
    def get_filename(self): return self.filename
    def get_title(self): return self.title
    def get_artist(self): return self.artist
    def set_id(self, id): self.id = id
    def get_id(self): return self.id

    def __str__(self):
        return str(self.get_id()) + "\n" \
               + self.get_title() + "\n" \
               + self.get_artist() + "\n"

class SongList:
    def __init__(self, rq):
        self.list = []
        self.files = {}
        self.rq = rq
        self.random = whrandom.whrandom()
        self.lock = Lock()

    def clear_playlist(self):
        self.list = []
        self.files = {}

    def add_song(self, server, filename, title, artist):
        if not self.files.has_key(filename):
            song = Song(len(self.list), server, filename, title, artist)
            self.files[filename] = song
            self.list.append(song)

    def clear_all_played(self):
        for x in self.list: x.set_played(0)

    def get_next(self):
        if not len(self.list): return None
        self.rq.lock.acquire()
        if self.rq.is_empty():
            self.rq.lock.release()
            first = self.random.randint(0, len(self.list) - 1)
            n = first
            while n != first and not self.list[n].was_played(): n += 1
            if n == first: self.clear_all_played()
            song = self.list[n]
            song.set_played(1)
            return song
        else:
            song = self.rq.get_next()
            self.rq.lock.release()
            return song

    def get_by_number(self, num):
        try:
            return self.list[num]
        except IndexError:
            return None

    def count(self): return len(self.list)

    def get_range(self, start, end):
        if start < 0: start = 0
        if end < 0: end = 0
        if end < start: end = start
        return copy.copy(self.list[start:end])

    def search(self, func):
        return [song for song in self.list if func(song)]
    
    def search_title(self, title):
        return self.search(lambda s, t=title: cifind(s.get_title(), t))
        
    def search_artist(self, artist):
        return self.search(lambda s, a=artist: cifind(s.get_artist(), a))

    def serialise_requests(self):
        self.rq.lock.acquire()
        l = [song.get_filename() for song in self.rq.get_queue()]
        self.rq.lock.release()
        return l

    def unserialise_requests(self, requests):
        self.rq.lock.acquire()
        for r in requests:
            if self.files.has_key(r): self.rq.add_end(self.files[r])
        self.rq.lock.release()

    def clear_requests(self):
        self.rq.lock.acquire()
        self.rq.clear()
        self.rq.lock.release()

        
